import { any, hasIn, is, path, pipe, split } from 'ramda';
var INDENTATION = 12;
var prop = pipe(split(/[[\].]/), path);
export var isListElement = function (val) {
    return is(Object, val) && !prop('key')(val);
};
export var isDictionaryElement = function (value) {
    return is(Object, value) &&
        hasIn('key', value) &&
        is(String, prop('key')(value));
};
export var isMetaList = function (value) {
    return is(Array, value) && any(isListElement, value);
};
export var isMetaDictionary = function (value) {
    return is(Array, value) && any(isDictionaryElement, value);
};
var stringify = function (space, val, narrow) {
    var padding = narrow ? '' : ' '.repeat(5);
    return JSON.stringify(val, null, space)
        .replace(/\\n/g, '\n')
        .replace(/}$/, padding + '}')
        .replace(/]$/, padding + ']');
};
export function normalizeIssueVariable(keyValue, value, indent, narrow) {
    var _a;
    var indentation = indent || INDENTATION;
    if (is(String, value)) {
        var padding_1 = narrow ? ' ' : ' '.repeat(10 + keyValue.length);
        var formattedValue = value
            .split(/\n/)
            .map(function (v, i) { return (i === 0 ? v : "" + padding_1 + v); })
            .join('\n');
        return "\"" + formattedValue + "\"";
    }
    if (isMetaList(value) || isMetaDictionary(value)) {
        var items = value.filter(Boolean).map(function (v) {
            if (is(String, v)) {
                return v;
            }
            var key = v.key, value = v.value;
            return key ? key + ": " + value : value;
        });
        return stringify(indentation, items, narrow);
    }
    if (isDictionaryElement(value)) {
        return stringify(keyValue.length, (_a = {}, _a[value.key] = value.value, _a), narrow);
    }
    if (isListElement(value)) {
        if (is(Array, value.value)) {
            return stringify(indentation, value.value);
        }
        return String(value.value);
    }
    return String(value);
}
